local BaikActionbar = LibStub("AceAddon-3.0"):GetAddon("BaikActionbar")
local Bags = BaikActionbar:NewModule("Bags")

-- Imports
local UiHelper = BaikActionbar.UiHelper
local State = BaikActionbar.State
local StateOpts = BaikActionbar.StateOpts
local StateMap = BaikActionbar.StateMap

-- Constants
local FRAMES = {
    MainMenuBarBackpackButton,
    CharacterBag0Slot,
    CharacterBag1Slot,
    CharacterBag2Slot,
    CharacterBag3Slot
}

-- Private Methods
local function SetHide()
    for _, frame in pairs(FRAMES) do
        UiHelper:Hide(frame)
    end
end

local function UnsetHide()
    for _, frame in pairs(FRAMES) do
        frame:SetParent("MainMenuBar")
    end
end

local function SetHover()
    local function SetVisible()
        for _, frame in pairs(FRAMES) do
            frame:SetAlpha(1.0)
        end
    end

    local function SetInivisble()
        for _, frame in pairs(FRAMES) do
            frame:SetAlpha(0.0)
        end
    end

    for _, frame in pairs(FRAMES) do
        frame:SetScript("OnEnter", SetVisible)
        frame:SetScript("OnLeave", SetInivisble)
        frame:SetAlpha(0.0)
    end
end

local function UnsetHover()
    for _, frame in pairs(FRAMES) do
        frame:SetScript("OnEnter", nil)
        frame:SetScript("OnLeave", nil)
        frame:SetAlpha(1.0)
    end
end

local SetState = {
    [State.hide] = SetHide,
    [State.hover] = SetHover,
    [State.show] = function() end
}

local UnsetState = {
    [State.hide] = UnsetHide,
    [State.hover] = UnsetHover,
    [State.show] = function() end
}

local function LoadConfig(value)
    local curState = StateMap[value]

    if curState then
        SetState[curState]()
    end
end

local bagGroup = {
    name = "Bag Options",
    type = "group",
    args = {
        bags = {
            name = "Visibility",
            type = "select",
            desc = "Adjust the bag bar visibility",
            values = StateOpts,
            get = function(info)
                return BaikActionbar.db.profile.bags
            end,
            set = function(info, key, value)
                -- Cache last state
                local curState = StateMap[BaikActionbar.db.profile.bags]
                local newState = StateMap[key]

                -- Update if state changed
                if newState ~= curState then
                    -- Remove last state configurations
                    if curState then
                        UnsetState[curState]()
                    end

                    -- Set new configurations and update key
                    LoadConfig(key)
                    BaikActionbar.db.profile.bags = key
                end
            end
        },
    }
}

-- Bags API
function Bags:OnUpdate()
    -- No action
end

-- Ace3 Callbacks
function Bags:OnInitialize()
    BaikActionbar.options.args["bagGroup"] = bagGroup
    BaikActionbar:DPrint("Bags OnInitialize")
end

function Bags:OnEnable()
    LoadConfig(BaikActionbar.db.profile.bags)
    BaikActionbar:DPrint("Bags OnEnable")
end

function Bags:OnDisable()
    BaikActionbar:DPrint("Bags OnDisable")
end
